use strict; 
use warnings; 
use URI;
use LWP::UserAgent;
use POSIX qw/ceil/;
use File::Basename;

### open and read the file 
open FILE_LINKS, $ARGV[0] or die $!;
binmode(FILE_LINKS, ":utf8");
my $file = do {local $/; <FILE_LINKS>};
close FILE_LINKS;

### retrieve parameters

## name, short description, long description
my $wp_name = ($file =~ m/div.id=\"wm_name\">(?:<img.*?>)?(.*?)<\/div>/g)[0];
my $wp_short = ($file =~ m/<strong>Quick.Description\:<\/strong>([\d\D]*?)<\/div>/g)[0];
$wp_short =~ s/^\s+//;
my $wp_long = ($file =~ m/(<div.id=(?:wm_longdesc|\"wm_variables\"|wm_loginstructions)>[\d\D]*?)<table.id=\"ctl00_ContentBody_WaymarkControl1_ExpandedWaymarkDetailsControl1_tblSearches/g)[0];

## fix "Created From" (eg. http://www.waymarking.com/waymarks/WM9XQC)
$wp_long =~ s|<div class="downloadbuttons">[\d\D]*?</div>||g;
$wp_long =~ s|<div class="downloadwrap">[\d\D]*?</div>|</div>|g;

## fix relative links for images
$wp_long =~ s|(src=[\"'])/|$1http://www.waymarking.com/|g;
$wp_long =~ s|(src=')\.\./|$1http://www.waymarking.com/|g;

## code, guid
my $wp_code = ($file =~ m/<strong>Waymark.Code\:<\/strong>.*?([\w\d]+?)<\/div>/g)[0];
my $wp_guid = ($file =~ m/http\:\/\/www\.waymarking\.com\/logs\/add\.aspx.*?guid=(.*?)[^\w\d-]/g)[0];

## placed date
my $wp_placed = ($file =~ m/<strong>Date.Posted\:<\/strong>.*?(\d.*?)<\/div>/g)[0];
$wp_placed =~ s/(\d+)\/(\d+)\/(\d+).(\d+)\:(\d+)\:(\d+).(\w)M/$3-0$1-0$2T0$4:0$5:0$6Z$7/g;
$wp_placed =~ s/([^\d])\d(\d\d)/$1$2/g;

if (substr($wp_placed,-1) eq "P")
{
  $wp_placed = substr($wp_placed,0,11) . (substr($wp_placed,11,2) + 12) . substr($wp_placed,13,8)
}

$wp_placed = substr($wp_placed,0,-1);

## placed by
my $wp_placed_by = ($file =~ m/<strong>Posted.by\:<\/strong>.*?profile\.aspx.*>(.*?)<\/a><\/a><\/div>/g)[0];

## lat, lon
my $wp_lat = ($file =~ m/http\:\/\/www\.geocaching\.com\/seek\/gmnearest\.aspx\?lat=(.*?).lng=(.*?)[^-\d\.]/g)[0];
my $wp_lon = ($file =~ m/http\:\/\/www\.geocaching\.com\/seek\/gmnearest\.aspx\?lat=(.*?).lng=(.*?)[^-\d\.]/g)[1];

## state, country
my $wp_st_ct = ($file =~ m/div.id=\"wm_location\"><strong>Location\:<\/strong>.*?(\w.*?)<\/div>/g)[0];

my $wp_country = $wp_st_ct;
my $wp_state = "";

if ($wp_st_ct =~ /,/)
{
  $wp_country = ($wp_st_ct =~ m/.*,.(.*)/g)[0];
  $wp_state = ($wp_st_ct =~ m/(.*),/g)[0];
}

## category
my $wm_category_guid = "";
my $wm_category_name = "";
my $wm_category_image = "";

if ($file !~ /div.id=\"wm_category\">not assigned a category yet<\/div>/)
{
  $wm_category_guid = ($file =~ m/div.id=\"wm_category\".*?guid\=(.*?)[^\w\d-]/g)[0];
  $wm_category_name = ($file =~ m/div.id=\"wm_category\".*?guid\=.*?>(.*?)\s*<\/a>/g)[0]; 
  $wm_category_image = "http://www.waymarking.com" . ($file =~ m/div.id=\"wm_name\">.*?src=.(.*?).>/g)[0];
}
                                   
## images
my @images = ($file =~ m/image[\d]+\.src=.http\:\/\/img\.groundspeak\.com\/waymarking\/display\/(.*?)[^\w\d\.-]/g);
my @images_log = ($file =~ m/image[\d]+\.src=.http\:\/\/img\.groundspeak\.com\/waymarking\/log\/display\/(.*?)[^\w\d\.-]/g);

# no rolling menu -> just use the main image
@images = ($file =~ m/rollimglink.*?http.*?display\/(.*?)[^\w\d\.-]/g) unless (@images);

## log
my @log_names = ($file =~ m/datalistitem-xs.*>(.*).(?:visited|wrote.comment.for).it<\/A><\/td>/g);
my @log_dates = ($file =~ m/datalistitem-xs.*?>(\d{1,2}\/\d{1,2}\/\d\d\d\d)<\/td>/g);
my @log_types = ($file =~ m/datalistitem-xs.*>.*(visited|wrote.comment.for).it<\/A><\/td>/g);

foreach (@log_dates)
{
  s/(\d+)\/(\d+)\/(\d+)/$3-0$1-0$2T08:00:00Z/g;
  
  # fix date to xx-xx-xxxx format
  s/([^\d])\d(\d\d)/$1$2/g;
}

### print info (debug)
print "Name: " . $wp_name . "\n___________________________\n";
print $wp_short . "\n___________________________\n";
#print $wp_long . "\n___________________________\n";
print "Placed: " . $wp_placed . "\n";
print "Placed By: " . $wp_placed_by . "\n";
print "Code: " . $wp_code . "\n";
print "GUID: " . $wp_guid . "\n";
print "Lat: " . $wp_lat . "\n";
print "Lon: " . $wp_lon . "\n";
print "Country: " . $wp_country . "\n";
print "State: " . $wp_state . "\n";
print "Category GUID: " . $wm_category_guid . "\n";
print "Category Name: " . $wm_category_name . "\n";
print "Category Image: " . $wm_category_image . "\n";

print "Image: " . $_ . "\n" foreach (@images);
print "Log Image: " . $_ . "\n" foreach (@images_log);
my $i = 0;
print "Log: " . $_ . " (" . $log_dates[$i++] . ")\n" foreach (@log_names);

### generate GPX file

my $gpx_file = substr($ARGV[0],0,-4) . ".gpx";

my $gpx = "<?xml version=\"1.0\" encoding=\"utf-8\"?>" . "\n";
$gpx .= "<gpx xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" version=\"1.0\" creator=\"Groundspeak Pocket Query\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd http://www.groundspeak.com/cache/1/0/1 http://www.groundspeak.com/cache/1/0/1/cache.xsd\" xmlns=\"http://www.topografix.com/GPX/1/0\">" . "\n";
$gpx .= " <desc>Geocache File</desc>" . "\n";  # original: This is an individual waymark generated from Waymarking.com
$gpx .= " <author>GPX Export Macro</author>" . "\n";

$gpx .= "<wpt lat=\"" . $wp_lat . "\" lon=\"" . $wp_lon . "\">" . "\n";
$gpx .= "  <time>" . $wp_placed . "</time>" . "\n";
$gpx .= "  <name>" . $wp_code . "</name>" . "\n";
$gpx .= "  <desc><![CDATA[" . $wp_name . "]]></desc>" . "\n";
$gpx .= "  <url>http://www.waymarking.com/waymarks/" . $wp_code . "</url>" . "\n";
$gpx .= "  <type><![CDATA[" . $wm_category_name . "]]></type>" . "\n";
$gpx .= "  <gsak:wptExtension xmlns:gsak=\"http://www.gsak.net/xmlv1/6\">" . "\n";
$gpx .= "   <gsak:UserData><![CDATA[" . $wm_category_name . "]]></gsak:UserData>" . "\n";
$gpx .= "   <gsak:User2><![CDATA[" . $wm_category_guid . "]]></gsak:User2>" . "\n";
$gpx .= "   <gsak:Guid>" . $wp_guid . "</gsak:Guid>" . "\n";
$gpx .= "  </gsak:wptExtension>" . "\n";
$gpx .= "  <groundspeak:cache available=\"True\" archived=\"False\" xmlns:groundspeak=\"http://www.groundspeak.com/cache/1/0/1\">" . "\n";
$gpx .= "  <groundspeak:name><![CDATA[" . $wp_name . "]]></groundspeak:name>" . "\n";
$gpx .= "  <groundspeak:placed_by><![CDATA[" . $wp_placed_by . "]]></groundspeak:placed_by>" . "\n";
$gpx .= "  <groundspeak:type>Waymark</groundspeak:type>" . "\n";
$gpx .= "  <groundspeak:container>Virtual</groundspeak:container>" . "\n";
$gpx .= "  <groundspeak:difficulty>1</groundspeak:difficulty>" . "\n";
$gpx .= "  <groundspeak:terrain>1</groundspeak:terrain>" . "\n";

## state, country
$gpx .= "   <groundspeak:country>" . $wp_country . "</groundspeak:country>" . "\n";
$gpx .= "   <groundspeak:state>" . $wp_state . "</groundspeak:state>" . "\n";

## short description (+ add WM category, which is used by MapyCZ makro)
$wp_short =~ s/\]\]/\]\]\]\]><!\[CDATA\[/g;
$gpx .= "   <groundspeak:short_description html=\"True\"><![CDATA[" . $wp_short . "<!--WM_Category:" . $wm_category_name . "-->]]></groundspeak:short_description>" . "\n";

## long description
$wp_long =~ s/\]\]/\]\]\]\]><!\[CDATA\[/g;
$gpx .= "   <groundspeak:long_description html=\"True\"><![CDATA[" . "\n";

# if we modify following, we maybe also need to change mapycz.js and mapycz.css
$gpx .= ("<span id=wm_cat_name><a href=\"http://www.waymarking.com/cat/details.aspx?f=1&st=2&guid=" . $wm_category_guid . "\"><img src=\"" . ($wm_category_image || "") . "\" border=\"0\"></a>" . "\n") if $wm_category_name;
$gpx .= "<span style='font-weight: bold; font-size: 21px;'>" . ($wm_category_name || "Not assigned a category yet") . "</span></span>" . "\n";

$gpx .= "<!-- category end -->" . "\n";
$gpx .= "<br><hr>" . "\n";
 
foreach (@images)
{
  $gpx .= "<img src=\"http://img.groundspeak.com/waymarking/display/" . $_ . "\">" . "\n";
#   $gpx .= "<img src=\"http://img.groundspeak.com/large/" . $a . "\">" . "\n";
#   $gpx .= "<img src=\"http://img.groundspeak.com/waymarking/" . $a . "\" width=100%>" . "\n";
}

$gpx .= "<!-- cache images end -->" . "\n";
$gpx .= "<br><hr>" . "\n";
$gpx .= $wp_long;
$gpx .= "<br><hr>" . "\n";
$gpx .= "\n<!-- log images start -->" . "\n";

foreach (@images_log)
{
  $gpx .= "<img src=\"http://img.groundspeak.com/waymarking/log/display/" . $_ . "\">" . "\n";
#   $gpx .= "<img src=\"http://img.groundspeak.com/log/large/" . $a . "\">" . "\n";
#   $gpx .= "<img src=\"http://img.groundspeak.com/log/" . $a . "\" width=100%>" . "\n";
}

$gpx .= "]]></groundspeak:long_description>" . "\n";

## logs
$gpx .= "   <groundspeak:logs>" . "\n";
$i = 0;

foreach $a (@log_names)
{
  $gpx .= "    <groundspeak:log id=\"" . (((convertWpt($wp_code) * 13) . "00") + $i) . "\">" . "\n";
  $gpx .= "    <groundspeak:date>" . $log_dates[$i] . "</groundspeak:date>" . "\n";
  $gpx .= "    <groundspeak:type>" . (($log_types[$i++] eq "visited") ? "Found it" : "Write note") . "</groundspeak:type>" . "\n";
  $a =~ s/\]\]/\]\]\]\]><!\[CDATA\[/g;
  $gpx .= "    <groundspeak:finder><![CDATA[" . $a . "]]></groundspeak:finder>" . "\n";
  $gpx .= "    <groundspeak:text encoded=\"False\"><![CDATA[]]></groundspeak:text>" . "\n";
  $gpx .= "   </groundspeak:log>" . "\n";  
}

$gpx .= "  </groundspeak:logs>" . "\n";
$gpx .= "</groundspeak:cache>" . "\n";
$gpx .= "</wpt>" . "\n";
$gpx .= "</gpx>" . "\n";

# print $gpx;
 
### save it back to the same file
open (MYFILE, '>' . $gpx_file);
binmode(MYFILE, ":utf8");
print MYFILE $gpx;
close (MYFILE);

# delete original html file
unlink $ARGV[0];

############

sub convertWpt 
{
  my $WPTCHARS = "0123456789ABCDEFGHJKMNPQRTVWXYZ";
  my $waypt = shift;
  
  return $waypt unless ($waypt =~ /^(?:GC|WM)([0-9A-HJ-KM-NP-RTV-Z]+)$/);
  my $wpt = $1;    
  return hex($wpt) if ($wpt =~ /^[0-9A-F]{1,4}$/); 

  my $accum = 0;
  $accum = $accum * 31 + index($WPTCHARS, $_) for (split('', $wpt));
  return abs($accum - 16 * 31 * 31 * 31 + 65536);
}